;/******************************/;
;/* Example For : JRHC08JL3 V1 */;
;/* Controller  : 68HC908JL3   */;
;/* Assembler   : CASM08Z.EXE  */;
;/* LCD Interface With PORT-A  */;
;/* RS-LCD = PTA4              */;
;/* EN-LCD = PTA5              */;
;/* D4-LCD = PTA0              */;
;/* D5-LCD = PTA1              */;
;/* D6-LCD = PTA2              */;
;/* D7-LCD = PTA3              */;
;/* RW,D0..D3 of LCD = GND     */;
;/******************************/;
;
;/*************************/;
;/* Register HC08 Equates */;
;/*************************/;
;
PORTA       EQU     $0000                   ; Port-A Data Register
PORTB       EQU     $0001                   ; Port-B Data Register
PORTD       EQU     $0003                   ; Port-D Data Regsiter
DDRA        EQU     $0004                   ; Port-A Direction Register
DDRB        EQU     $0005                   ; Port-B Direction Register
DDRD        EQU     $0007                   ; Port-D Direction Register
PDCR        EQU     $000A                   ; Port-D Control Register
PTAPUE      EQU     $000D                   ; Port-A Pull-Up Enable
KBSCR       EQU     $001A                   ; Keyboard Status Control
KBIER       EQU     $001B                   ; Keyboard Interrupt Enable
INTSCR      EQU     $001D                   ; IRQ Status Control
CONFIG2     EQU     $001E                   ; Config Register2
CONFIG1     EQU     $001F                   ; Config Register1
TSC         EQU     $0020                   ; TIM Status & Control
TCNTH       EQU     $0021                   ; TIM Counter High
TCNTL       EQU     $0022                   ; TIM Counter Low
TMODH       EQU     $0023                   ; TIM Counter Modulo High
TMODL       EQU     $0024                   ; TIM Counter Modulo Low
TSC0        EQU     $0025                   ; TIM CH0 Status & Control
TCH0H       EQU     $0026                   ; TIM CH0 High
TCH0L       EQU     $0027                   ; TIM CH0 Low
TSC1        EQU     $0028                   ; TIM CH1 Status & Control
TCH1H       EQU     $0029                   ; TIM CH1 High
TCH1L       EQU     $002A                   ; TIM CH1 Low
ADSCR       EQU     $003C                   ; ADC Status & Control
ADR         EQU     $003D                   ; ADC Data Read
ADICLK      EQU     $003E                   ; ADC Input Clock
BSR         EQU     $FE00                   ; Break Status Register
RSR         EQU     $FE01                   ; Reset Status Register
BFCR        EQU     $FE03                   ; Break Flag Control
INT1        EQU     $FE04                   ; Interrupt Status1
INT2        EQU     $FE05                   ; Interrupt Status2
INT3        EQU     $FE06                   ; Interrupt Status3
FLCR        EQU     $FE08                   ; Flash Control Register
FLBPR       EQU     $FE09                   ; Flash Block Protect
BRKH        EQU     $FE0C                   ; Break Address High
BRKL        EQU     $FE0D                   ; Break Address Low
BRKSCR      EQU     $FE0E                   ; Break Status & Control
COPCTL      EQU     $FFFF                   ; Reset COP
;/* End Equate I/O Register of HC08 */;
;
;/********************/;
;/* Register Equates */;
;/********************/;
;
PORT_LCD    EQU     PORTA                   ; PORTA Control LCD
RS_LCD      EQU     04H                     ; PTA4
EN_LCD      EQU     05H                     ; PTA5
D4_LCD      EQU     00H                     ; PTA0
D5_LCD      EQU     01H                     ; PTA1
D6_LCD      EQU     02H                     ; PTA2
D7_LCD      EQU     03H                     ; PTA3

            ORG     80H
DLY_BUFF    DS      1                       ; Delay Buffer
$BASE       10T                             ; Defualt Number to Decimal

            ;/****************************/;
            ;/* 68HC908JL3 Port I/O Test */;
            ;/****************************/;
            ;
            ORG     $EC00                   ; Start Main (68HC908JL3)
            ;
RESET:      BSET    0,CONFIG1               ; Disable COP Watch-Dog
            LDA     #%00111111              ; Port PA5..0 = Output
            STA     DDRA
            BCLR    EN_LCD,PORT_LCD         ; Initial Control LCD
            BCLR    RS_LCD,PORT_LCD
            BCLR    D7_LCD,PORT_LCD
            BCLR    D6_LCD,PORT_LCD
            BSET    D5_LCD,PORT_LCD
            BSET    D4_LCD,PORT_LCD
            ;
            ;/**************************/;
            ;/* Power-on Delay For LCD */;
            ;/**************************/;
            LDHX    #0000H                  ; Power-On Delay Count
PWR_DLY0:   AIX     #1                      ; H:X + 1
            CPHX    #0FFFFH
            BNE     PWR_DLY0
            JSR     INIT_LCD                ; Initial LCD
            ;
LOOP:       LDA     #$00                    ; Cursor Line1 (16x2)
            JSR     GOTO_LCD
            JSR     PRINT_LCD
            DB      'EXAMPLE LCD 16x2',00H
            LDA     #$40                    ; Cursor Line2 (16x2)
            JSR     GOTO_LCD
            JSR     PRINT_LCD
            DB      'JR-68HC908JL3 V1',00H
            ;
            MOV     #250,DLY_BUFF           ; Delay 1-Second
            JSR     DLY_MSEC
            MOV     #250,DLY_BUFF
            JSR     DLY_MSEC
            MOV     #250,DLY_BUFF
            JSR     DLY_MSEC
            MOV     #250,DLY_BUFF
            JSR     DLY_MSEC
            ;
            LDA     #$00                    ; Cursor Line1 (16x2)
            JSR     GOTO_LCD
            JSR     PRINT_LCD
            DB      'JR-68HC908JL3 V1',00H
            LDA     #$40                    ; Cursor Line2 (16x2)
            JSR     GOTO_LCD
            JSR     PRINT_LCD
            DB      'EXAMPLE LCD 16x2',00H
            ;
            MOV     #250,DLY_BUFF           ; Delay 1-Second
            JSR     DLY_MSEC
            MOV     #250,DLY_BUFF
            JSR     DLY_MSEC
            MOV     #250,DLY_BUFF
            JSR     DLY_MSEC
            MOV     #250,DLY_BUFF
            JSR     DLY_MSEC
            JMP     LOOP

;/********************/;
;/* Initial Char LCD */;
;/* 4-Bit Interface  */;
;/* RS    = Bit-2    */;
;/* EN    = Bit-3    */;
;/* D4..7 = Bit-4..7 */;
;/********************/;
;
INIT_LCD:   LDA     #33H                    ; DL = 1 ,DL =1
            JSR     WR_INS
            LDA     #32H                    ; DL = 1 ,DL =0
            JSR     WR_INS
            LDA     #28H                    ; Function set
            JSR     WR_INS                  ; DL=0 4Bit,N=1 2Line,F=0 5X7
            LDA     #0CH                    ; Display on/off Control
            JSR     WR_INS                  ; Entry display,cursor off,not blink
            LDA     #06H                    ; Entry mode set
            JSR     WR_INS                  ; I/D=1 Increment,S=0 cursor shift
CLR_LCD:    LDA     #01H                    ; Clear Display
            JSR     WR_INS                  ; Clear Display,Set DD RAM addr=0
            RTS

;/**********************/;
;/* Enable Pin EN CLCD */;
;/* Pin EN LCD = Bit-3 */;
;/* Active Chip select */;
;/**********************/;
;
ENA_LCD:    JSR     BUSY                    ; Busy Delay
            BSET    EN_LCD,PORT_LCD         ; Enable Chips Select
            ;
            PSHX                            ; 2 Cycle
            LDX     #25                     ; 2 Cycle
ENA_LCD0:   NOP                             ; 1 Cycle
            DBNZX   ENA_LCD0                ; 3 Cycle (25x4 =100)
            PULX                            ; 2 Cycle
            ;
            BCLR    EN_LCD,PORT_LCD         ; Disable Chips Select
            RTS

;/*******************/;
;/* Busy CLCD Delay */;
;/* Defualt = 100uS */;
;/* Oscilator 32MHz */;
;/* 1Cycle = 1/8MHz */;
;/* 100uS = 800Cycle*/;
;/*******************/;
;
BUSY:       PSHX                            ; 2 Cycle
            LDX     #198                    ; 2 Cycle
BUSY1:      NOP                             ; 1 Cycle
            DBNZX   BUSY1                   ; 3 Cycle (198x4 =792)
            PULX                            ; 2 Cycle
            RTS                             ; 4 Cycle

;/**********************/;
;/* Write ASCII to LCD */;
;/* Input : ACC(ASCII) */;
;/**********************/;
;
WR_LCD:     PSHA                            ; Save Data Low Byte
            AND     #$F0                    ; High Byte
            NSA                             ; Nibble Swap A
            STA     PORT_LCD
            BSET    RS_LCD,PORT_LCD         ; Set RS For Write Data Select
            JSR     ENA_LCD
            PULA                            ; Restore Data Low Byte
            ;
            AND     #$0F                    ; Low Byte
            STA     PORT_LCD
            BSET    RS_LCD,PORT_LCD         ; Set RS For Write Data Select
            JSR     ENA_LCD
            RTS

;/**************************/;
;/* Write Instruction LCD  */;
;/* Input  : ACC (Command) */;
;/* Output : Data bus LCD  */;
;/**************************/;
;
WR_INS:     PSHA                            ; Save Data Low Byte
            AND     #$F0                    ; High Byte
            NSA                             ; Nibble Swap A
            STA     PORT_LCD
            BCLR    RS_LCD,PORT_LCD         ; Clear RS For Write Instruction
            JSR     ENA_LCD
            PULA                            ; Restore Data Low Byte
            ;
            AND     #$0F                    ; Low Byte
            STA     PORT_LCD
            BCLR    RS_LCD,PORT_LCD         ; Clear RS For Write Instruction
            JSR     ENA_LCD
            MOV     #2,DLY_BUFF             ; Wait Command Ready = 2mS
            JSR     DLY_MSEC
            RTS

;/************************/;
;/* Goto position of LCD */;
;/* Input : ACC (addr.)  */;
;/************************/;
;
GOTO_LCD:   ORA     #$80                    ; Set ACC.7
            JSR     WR_INS
            RTS

;/*******************/;
;/* Delay Time 1 mS */;
;/* CPU Bus = 8 MHz */;
;/* 1Cycle = 1/8MHz */;
;/* 1mS = 8000Cycle */;
;/* Input :DLY_BUFF */;
;/*******************/;
;
DLY_MSEC:   PSHH                            ; Save Register (2 Cycle)
            PSHX                            ; 2 Cycle
            PSHA                            ; 2 Cycle
            TPA                             ; CCR -> A (1 Cycle)
            PSHA                            ; 2 Cycle
            LDA     DLY_BUFF                ; 3 Cycle (Get Delay)
            ;
DLY_MS0:    LDHX    #886                    ; 3 Cycle
DLY_MS1:    NOP                             ; 1 Cycle
            AIX     #-1                     ; H:X - 1 (2 Cycle)
            CPHX    #0                      ; 3 Cycle
            BNE     DLY_MS1                 ; 3 Cycle ( 9 x 886 = 8000 Cycle)
            DBNZA   DLY_MS0                 ; 3 Cycle
            ;
            PULA                            ; 2 Cycle
            TAP                             ; A -> CCR (2 Cycle)
            PULA                            ; 2 Cycle
            PULX                            ; 2 Cycle
            PULH                            ; 2 Cycle
            RTS                             ; 4 Cycle

;/********************************/;
;/* Print ASCII Data to LCD Port */;
;/* Usage    : JSR  PRINT_LCD    */;
;/*          : DB   'xxxxxxxx'   */;
;/*          : DB   $00          */;
;/* Note     : Last Byte = $00   */;
;/********************************/;
;
PRINT_LCD:  PULH                        ; Pop Stack PCH
            PULX                        ; Pop Stack PCL
            PSHA                        ; Save ACC
            ;
PRN_LCD1:   LDA     0,X                 ; Get Data For Print
            BEQ     PRN_LCD2            ; If Last Byte Jump
            JSR     WR_LCD              ; Write Data to LCD
            INCX
            BRA     PRN_LCD1
            ;
PRN_LCD2:   INCX                        ; Skip Last Byte(00)
            ;
            PULA                        ; Restore ACC
            PSHX                        ; Push PCL to Stack
            PSHH                        ; Push PCH to Stack
            RTS

            ;/***************/;
            ;/* User Vector */;
            ;/***************/;
            ;
            ORG     $FFF6               ; Security Code
            DB      'JR08JLV1'
            ORG     $FFFE               ; Reset Vector
            DW      RESET

            END

